using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;
using Serilog;
using ShoeStore.Repositories;
using ShoeStore.Repositories.Implementations;
using Unity;
using Unity.Lifetime;
using Unity.Microsoft.Logging;

namespace ShoeStore
{
    internal static class Program
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            // To customize application configuration such as set high DPI settings or default font,
            // see https://aka.ms/applicationconfiguration.
            ApplicationConfiguration.Initialize();
            Application.Run(CreateContainer().Resolve<FormStorage>());
        }
        private static UnityContainer CreateContainer()
        {
            var container = new UnityContainer();

            container.AddExtension(new
                LoggingExtension(CreateLoggerFactory()));

            container.RegisterType<IConnectionString,
            ConnectionString>(new SingletonLifetimeManager());
            container.RegisterType<IFactoryRepository,
            FactoryRepository>(new TransientLifetimeManager());
            container.RegisterType<IStoreRepository,
            StoreRepository>(new TransientLifetimeManager());
            container.RegisterType<IShoesRepository, ShoesRepository>(new
            TransientLifetimeManager());
            container.RegisterType<IShoesSaleRepository,
            ShoesSaleRepository>(new TransientLifetimeManager());
            container.RegisterType<IShoesReplenishmentRepository,
            ShoesReplenishmentRepository>(new TransientLifetimeManager());
            return container;
        }
        private static LoggerFactory CreateLoggerFactory()
        {
            var loggerFactory = new LoggerFactory();
            loggerFactory.AddSerilog(new LoggerConfiguration()
            .ReadFrom.Configuration(new ConfigurationBuilder()
            .SetBasePath(Directory.GetCurrentDirectory())
            .AddJsonFile("appsettings.json")
            .Build())
            .CreateLogger());
            return loggerFactory;
        }
    }
}
