﻿using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using ShoeStore.Entities;
using ShoeStore.Entities.Enums;
using System;
using System.Collections.Generic;
using System.Data.SqlClient;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ShoeStore.Repositories.Implementations;

internal class ShoesReplenishmentRepository : IShoesReplenishmentRepository
{
    private readonly IConnectionString _connectionString;
    private readonly ILogger<ShoesReplenishmentRepository> _logger;
    public ShoesReplenishmentRepository(IConnectionString connectionString,
    ILogger<ShoesReplenishmentRepository> logger)
    {
        _connectionString = connectionString;
        _logger = logger;
    }
    public void CreateShoesReplenishment(ShoesReplenishment shoesReplenishment)
    {
        _logger.LogInformation("Добавление объекта");
        _logger.LogDebug("Объект: {json}",
        JsonConvert.SerializeObject(shoesReplenishment));
        try
        {
            using var connection = new
            NpgsqlConnection(_connectionString.ConnectionString);
            connection.Open();
            using var transaction = connection.BeginTransaction();
            var queryInsert = @"
INSERT INTO ShoesReplenishments (factorysid, DateReceipt)
VALUES (@factorysid, @DateReceipt);
SELECT MAX(Id) FROM ShoesReplenishments";
            var shoesReplenishmentId =
            connection.QueryFirst<int>(queryInsert, shoesReplenishment, transaction);
            var querySubInsert = @"
INSERT INTO ShoesShoesReplenishments (shoesreplenishmentsid, ShoesId, Count)
VALUES (@ShoesReplenishmentId,@ShoesId, @Count)";
            foreach (var elem in shoesReplenishment.ShoesShoesReplenishments)
            {
                connection.Execute(querySubInsert, new
                {
                    shoesReplenishmentId,
                    elem.ShoesId,
                    elem.Count
                }, transaction);
            }
            transaction.Commit();
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при добавлении объекта");
            throw;
        }
    }
    public void DeleteShoesReplenishment(int id)
    {
        _logger.LogInformation("Удаление объекта");
        _logger.LogDebug("Объект: {id}", id);
        try
        {
            using var connection = new
            NpgsqlConnection(_connectionString.ConnectionString);
            var queryDelete = @"
DELETE FROM ShoesReplenishments
WHERE Id=@id";
            connection.Execute(queryDelete, new { id });
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при удалении объекта");
            throw;
        }
    }
    public IEnumerable<ShoesReplenishment> ReadShoesReplenishment(DateTime?
    dateForm = null, DateTime? dateTo = null, int? shoesId = null, int? factorysId =
    null)
    {
        _logger.LogInformation("Получение всех объектов");
        try
        {
            using var connection = new
            NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = @"SELECT * FROM ShoesReplenishments";
            var shoesReplenishments =
            connection.Query<ShoesReplenishment>(querySelect);
            _logger.LogDebug("Полученные объекты: {json}",
            JsonConvert.SerializeObject(shoesReplenishments));
            return shoesReplenishments;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при чтении объектов");
            throw;
        }
    }
}
